; Process all cubes in a given directory
; list - optional list of file names
pro allcubes5, par, flat=flat, list=list

;   files = findfile(par.dirdat+'*CapObj.fits', count=n)
  files = findfile(par.dirdat+'*_cube.fits', count=n)

   print, n, ' files found'
   if keyword_set(list) then begin
      print, 'Filtering by list, N= ', n_elements(list)
   endif   
   
stop
rmin = 9. ; min. ring radius, pixels, hard-coded
rmax = 15.  ; max ring radius

mmax = par.mmax
nsect = par.nsect

if keyword_set(flat) then begin
  ff = findfile(par.dirdat+'flat.fits', count=nf)
  if (ff eq 0) then begin
     print, 'File flat.fits is not found, proceed without'
     flat=1.
  end else flat = readfits(par.dirdat+'flat.fits') 
endif ; flat

impar = {backgr:0.,flux:0.,fluxvar:0.,rad:0.,rwidth:0., $
       xc:0.,yc:0.,xcvar:0.,ycvar:0.,coma:0.,angle:0., $
       contrast:0.,noisepar:fltarr(4)}

; impar is returned by cubecoef.
; in dat, flux is in el, anoise - var. of norm. coef,  rnoise in pixels rms
; rnoise1 - analytic diff. rad. variance, pix^2. 

  el={fname:'',jd:0D0,hr:0,bv:0.,zen:0.,texp:1e-3,gain:0.,nx:0,nz:0, impar:impar, $
  flux:0.,rad:0.,rrms:0.,pow:fltarr(mmax+1), cov:fltarr(mmax+1), anoise:0.,rnoise:0., $
   prof:fltarr(8), see:0.,fsee:0.,noise:0.,scint:0.,rmserr:0.,see2:0.,rnoise1:0.,wind:0.,tau0:0.}

   dat = replicate(el,n)

   k=0
   for i=0,n-1 do begin
     fname = strmid(files[i],strlen(par.dirdat))
     fname = strmid(fname, 0, strlen(fname)-5)
; name in the list?
   if keyword_set(list) then begin
      s = where(fname eq list, ns)
      if (ns eq 0) then continue
   endif   
    
;     stop
     yy = fix(strmid(fname,0,4))
     mm = fix(strmid(fname,5,2))
     dd = fix(strmid(fname,8,2))
     hh = fix(strmid(fname,11,2))
     min = fix(strmid(fname,13,2))
     sec =  fix(strmid(fname,15,2))
     
     jdcnv, yy, mm, dd, hh+min/60. + sec/3600., jd
     dat[k].jd = jd - 2400000.D0
     dat[k].fname = fname
     dat[k].texp=1e-3  ; hard-coded
 
     cube = readfits(files[i], hd)
     if (n_elements(cube) lt 2) then continue
     sz =size(cube)
     nz = sz[3]
     if (sz[0] ne 3) then continue ; not a cube
     if (nz lt 100) then continue ; too short cube
     gain = sxpar(hd, 'gain')
     dat[k].gain = gain
     dat[k].nx = sz[1]
     dat[k].nz = sz[3]
     star = sxpar(hd, 'STAR')
     dat[k].hr = fix(star) ; HR number
;     stop, 'Ready for cube crunching!'

;     cubecoef, cube, par, coef, impar, imav=imav, /displ

    if keyword_set(flat) then $
     cubecoef, cube, par, coef, impar, imav=imav, flat=flat $
       else cubecoef, cube, par, coef, impar, imav=imav

     if (impar.flux lt 0) then continue ; empty cube?

     save, file=par.dirdat+fname+'.idl', coef, impar  ; save coefficients
     
     ringrad = mean(coef[0:nsect-1,*])     
     if ((ringrad lt rmin) or (ringrad gt rmax)) then continue ; ring too small

;     stop, 'Ready to save average image'
     writefits, par.dirdat+fname+'-av.fits', imav, hd
     
     eladu = 3.60*10^(-gain/200.)
;     eladu = 0.64*10^( (150 - gain)/200.)
	
     flux = total(coef[nsect:2*nsect-1,*])/nz  ; total flux in ADU
     dat[k].flux = flux*eladu ; flux in electons per frame

     dat[k].impar = impar
     dat[k].rad = ringrad

     statmom,  coef, par, pow, rrms, rnoise, cov=cov
     
     dat[k].rrms = rrms
     dat[k].rnoise = rnoise
     dat[k].pow = pow
     dat[k].cov = cov
;     stop
     k++
   endfor
print, 'Done, valid cubes: ',k
dat = dat[0:k-1]
save, file=par.datfile, dat

;stop
end
;---------------------------------------------------------------------
; compute zenith distance using BS catalog, JD, and site coordinates
; see speckle/getlog2.pro => getzen
pro getzen, par

; lonrad = par.lon/!radeg ; site longitude, radian
latrad = par.lat/!radeg ; latitude

restore, 'bsc.idl' ; BS catalog, full
; restore, 'bscb.idl' ; BS catalog, bright stars only

restore, par.datfile ; dat
n = n_elements(dat)
allha = fltarr(n) ; hour angles
kmiss=0 ; stars without HR numbers

for i=0,n-1 do begin
  if (dat[i].hr eq 0) then begin
     kmiss++
     continue ; skip
  endif   
  s = where(dat[i].hr eq bsc.hr, ns) ; find in BSC
  if (ns ne 1) then begin
    kmiss++
    continue ; skip
  endif   
  dat[i].bv = bsc[s].bv ; fetch color
  ra = bsc[s].radeg/!radeg ; RA, radian
  dec = bsc[s].dedeg/!radeg ; DEC, radian
  
  CT2LST, lst, par.lon, dummy, dat[i].jd+2400000D0 ; local siderial time, hours
  ha = lst*!dpi/12. - ra  ;  hour angle, radian
  allha[i] = (ha*!radeg/15) mod 24. ; ha, hours 
  sinalt =  sin(latrad)*sin(dec) + cos(latrad)*cos(dec)*cos(ha)
  zen = 90. - !radeg*asin(sinalt) ; zenith angle
  dat[i].zen = zen 
endfor
print, kmiss, ' non-identified objects'

plot, dat.zen, xtitle='Sequantial number', ytitle='Zenith distance'

; stop
save, file=par.datfile, dat

end
;---------------------------------------------------------------------
;---------------------------------------------------------------------
; compute noise
pro noisecubes, par

restore, par.datfile ; dat
n = n_elements(dat)
altnoise = fltarr(n)

for i=0,n-1 do begin
   impar = dat[i].impar
   flux = dat[i].flux
   ; noise on angular spectrum
   anoise = impar.noisepar[0]/flux + impar.noisepar[1]*(par.ron/flux)^2
   ; noise on diff. radius, in pixels
   flux = flux/par.nsect ; flux per sector
   rnoise =  2.*(impar.noisepar[2]/flux + impar.noisepar[3]*(par.ron/flux)^2/par.nsect)
   dat[i].anoise = anoise
   dat[i].rnoise1 = rnoise ; variance, pix^2
   altnoise[i] = total(dat[i].pow[par.mmax-3:par.mmax-1])/3. ; variance of large-m coefficients
endfor

save, fi=par.datfile, dat
print, 'Noise is computed'

amax = max(altnoise)*1.5
plot, dat.anoise, altnoise/dat.anoise, psym=6, xtitle='Analytic noise', ytitle='Real/analytic'

oplot, [0,amax], [1,1], li=1


;plot, sqrt(dat.rnoise1), dat.rnoise^2/dat.rnoise1, psym=6,  xtitle='Analytic rms noise [pix]', ytitle='Real/analytic radius variance'
;oplot, [0,1], [1,1], li=1

;stop

end
;---------------------------------------------------------------------
; filter by centroids
pro filterdat, par

dmax = 18 ; hard-coded max offset

restore, par.datfile
s = where((abs(dat.impar.xc) lt dmax) and (abs(dat.impar.yc) lt dmax), ns)
n = n_elements(dat)

print, 'Original, filtered: ', n, ns
print, '.C to save...'

dat= dat[s]
; stop

save, fi=par.datfile, dat
print, 'Saved filtered in '+par.datfile
end
;---------------------------------------------------------------------
;---------------------------------------------------------------------
; Listing
pro listcubes5, par

restore, par.datfile ; dat
;restore, filename ; dat

n = n_elements(dat)
print, n, ' records found'

close, /all
openw, 11, 'listcubes.txt'

; Fname HR Zen flux rad see2 see fsee scint noise rmserr
fmt='(A-20,I5,F7.1, E10.3,F7.2, 3F6.2, 2X, F7.3, F6.3)'
printf, 11, '#File HR Zen Flux  Rad See2 See Fsee Scint rmsprof'

for i=0, n-1 do begin
  el =dat[i]
  name = strmid(el.fname,0,17)
;  stop
  print,      name,el.hr,el.zen,el.flux,   el.rad,el.see2,el.see,el.fsee,el.scint,el.rmserr, format=fmt
  printf, 11, name,el.hr,el.zen,el.flux,   el.rad,el.see2,el.see,el.fsee,el.scint,el.rmserr, format=fmt
  
endfor
print, 'DONE'
close, 11
end
;---------------------------------------------------------------------
;---------------------------------------------------------------------
;---------------------------------------------------------------------
