; 2022-05-03 Create .dat structure from .stm file.
; see readstm.pro and allcubes5.pro as examples
pro allstm, par, tag=tag

close, /all
openr, 11, par.stmfile

rmin = 9. ; min. ring radius, pixels, hard-coded
rmax = 15.  ; max ring radius
mmax = par.mmax
nsect = par.nsect
eladu = 0.36  ; for G=200 default electrons per ADU
; eladu = 3.60*10^(-gain/200.)

if not keyword_set(tag) then begin ; average lines by default
   tag = 'M'
   k0=4
endif else begin 
   k0=3 
endelse

a = ''
; readf, 11, a  ; first line
nmax= 10000  ; max number of records

impar = {backgr:0.,flux:0.,fluxvar:0.,rad:0.,rwidth:0., $
       xc:0.,yc:0.,xcvar:0.,ycvar:0.,coma:0.,angle:0., $
       contrast:0.,noisepar:fltarr(4)}

 el={fname:'',jd:0D0,hr:0,bv:0.,zen:0.,texp:1e-3,gain:0.,nx:64,nz:2000, impar:impar, $
  flux:0.,rad:0.,rrms:0.,pow:fltarr(mmax+1), cov:fltarr(mmax+1), anoise:0.,rnoise:0., $
   prof:fltarr(8), see:0.,fsee:0.,noise:0.,scint:0.,rmserr:0.,see2:0.,rnoise1:0.,wind:0.,tau0:0.}

   dat = replicate(el,nmax)

k = 0  ; record counter
while not eof(11) do begin
readf, 11, a

b = strsplit(a, ',', /extract)
n = n_elements(b)
;print, n, ' tokens in the line'
label = b[0]  ; tag  datetime


if (label ne tag) then continue ; only labeled lines


datetime = strmid(b[1],1) ; skip first space
star = fix(b[2])

     yy = fix(strmid(datetime,0,4))
     mm = fix(strmid(datetime,5,2))
     dd = fix(strmid(datetime,8,2))
     hh = fix(strmid(datetime,11,2))
     min = fix(strmid(datetime,14,2))
     sec = fix(strmid(datetime,17,2))    

     jdcnv, yy, mm, dd, hh+min/60. + sec/3600., jd
     dat[k].jd = jd - 2400000.D0
     dat[k].texp=1e-3  ; hard-coded

     impar1 = float(b[k0:k0+11])
     rad = impar1[3]
     if ((rad lt rmin) or (rad gt rmax)) then continue ; bad data

     print, datetime

     dat[k].fname = datetime
     dat[k].hr = star
     ; noisepar = float(b[k0+12:k0+15])
     for j=0,11 do dat[k].impar.(j) = impar1[j]
     dat[k].impar.noisepar = float(b[k0+12:k0+15])
     m = mmax+1
     k1 = k0+16
     dat[k].pow = float(b[k1:k1+m-1])
     dat[k].cov =  float(b[k1+m:k1+2*m-1])
     dat[k].flux = impar1[1]*eladu  ; flux in electrons
     dat[k].rad = rad

     k2 = k1+2*m  ; 59  

    dat[k].rnoise = float(b[k2])
    dat[k].rrms = sqrt(float(b[k2+1]))
;    mcoef = float(b[k2+2:k2+2+13])

     plot, dat[k].pow, /ylog
     oplot, dat[k].cov, li=2

;     stop
    k++
endwhile
close, 11 
print, k, ' records found'
dat = dat[0:k-1]

save, fi=par.datfile, dat

; stop


end
