; Generate undistorted image from par-file
; modeled after ringsim.pro. Compile ztools if aberrations are given
pro getimage, par, impix, zn=zn,zrad=zrad, imh1=imh1, rad=rad

D = par.d
EPS = par.eps
pdist = par.pdist
lambda = par.lambda

ngrid = 256

; Select over-sampling and grid size
d1 = lambda/(par.pixel)*206265. ; pupil scale matching pixels
npixperpix =  2^(floor(alog(1.5*d/d1)/alog(2.))+2) ; over-sampling
asperpix = par.pixel/npixperpix ; fine sampling in arcseconds

nccd = floor(2*ngrid/npixperpix) ; CCD image size
size = d1*npixperpix

; asperpix = 206265.*lambda / size  ; same 
pixel = size/(2.*ngrid)  ; spatial pixel

ringradpix = 0.93*d*(1 + eps)/(4.*pdist)*206265./asperpix ; 
; print, 'Nominal ring radius [pix, arcsec]:', ringradpix/npixperpix, ringradpix*asperpix
if (ringradpix gt 0.8*ngrid) then begin
   print, 'Ring radius too big, returning'
   return
endif   

apert = fltarr(2*ngrid,2*ngrid)
r = shift(dist(2*ngrid), ngrid, ngrid)  ; radial distance, pixels
Radpix = d*0.5/pixel
inside = where((r le Radpix) and (r ge Radpix*eps), nn)
if (nn gt 0) then apert(inside) = 1.
; Add phase distortions: defocus and spherical. a4 negative for intra-focal.

a4 = d^2/(lambda*pdist) *(!pi/8/sqrt(3.)) ; Zernike in radians
a11 = -0.1*a4  ; matching spherical
tmp = a11*sqrt(5.)*(6.*(r/Radpix)^4 - 6.*(r/Radpix)^2)
tmp += a4*2.*sqrt(3.)*((r/Radpix)^2 - 0.5)
; print, 'Nonimal a4,a11 [rad]: ',a4,a11


; Add Zernike aberrations if specified, use ztools. See sim
if keyword_set(zn) then begin
  x = (findgen(2*ngrid) - ngrid) # replicate(1., 2*ngrid)
  theta = atan(transpose(x),x)
  ninside = n_elements(inside)
  zgrid = fltarr(ninside,2)
  zgrid[*,0] = r(inside)/Radpix
  zgrid[*,1] = theta(inside)
  ; -- use Ztools.pro here! ---------------------
  phase = zgrid[*,0]*0. ; empty array
  for k=0,n_elements(ZRAD)-1 do phase += ZRAD[k]*zernike_estim(zn[k], zgrid)
  tmp[inside] += phase
endif

uampl = complex(cos(tmp), sin(tmp))*apert

; undistorted image in fine pixels
imh1 = abs(shift(fft(shift(uampl,ngrid,ngrid),/inverse),ngrid, ngrid))^2
rring = total(imh1*r)/total(imh1)

rad =rring*asperpix  ; ring radius in arcseconds

; print, 'True ring radius [pix,arcsec]: ', rring/npixperpix, rring*asperpix

impix = rebin(shift(imh1,npixperpix/2,npixperpix/2),nccd,nccd)

;impix = rebin(imh1,nccd,nccd)

impix = impix/total(impix) 
; tvscl, congrid(impix, 256, 256)

;stop

end
;----------------------------------------------------------------------------
pro test


par = readpar('mar21a.par') ; nominal parameters
pdist = [0.25, 0.3, 0.4, 0.6, 0.8, 1.]*1e3 ; propagation distance
n = n_elements(pdist)
radii = fltarr(n)

for i=0,n-1 do begin
   par.pdist = pdist[i]
   getimage, par, impix, rad=rad
   radii[i] = rad
endfor

; Geometric ring radius in arcseconds
;radnom =  0.9*par.d*(1 + par.eps)/(4.*pdist)*206265. ; for quadratic subtaction
radnom =  0.93*par.d*(1 + par.eps)/(4.*pdist)*206265. ; no subtaction


rh = pdist*radii  ; RH in m.arcsec
rhnom = pdist*radnom

;rrel = rh/rh[0] -1
;plot, pdist, rrel

;plot, pdist, rh
;oplot, pdist, rhnom, li=2

plot, pdist, radii/radnom-1 

stop

; Correct by quadratic subtraction of 3*lam/D

r0 = 3.5*par.lambda/par.d*206265 ; full-aperture diffraction radius, arcsec
rh1 = pdist*sqrt(radii^2 - r0^2)
rrel1 = rh1/rh1[0] - 1
;plot, pdist, rrel1

plot, pdist, rh1/rhnom - 1

stop

end
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------


