; Read .SER image cube format 
; for ASICap files, information from text can be added by the /text
; keyword, usind rdtext.pro

pro rdser, filename, img, timestamp=timestamp, fits=fits, text=text

close, /all
openr, 11, filename  ;, /xdr

preamble = bytarr(14)
par = lonarr(7)
postamble = bytarr(178 - 14 - 4*7)

; Here it crashes if the file is corrupted - need to check the length.
readu, 11, preamble, par, postamble

; the integers are little-endian, lSB first
; bytorder, 4byte, /lswap  converts 4-byte array to big-endian

nx = par[3]
ny = par[4]
nz = par[6]

print, string(preamble)+' ==> ',nx,ny,nz
;print, string(preamble)+' ==> ',nx,ny,nz

if (par[5] eq 16) then img = uintarr(nx,ny,nz) else img = bytarr(nx,ny,nz)

; postabmle[120:127] is 8=byte datetime, [128:135] = UTC date
; stop

readu, 11, img

if (par[5] eq 16) then img = img/16  ; shift 4-bit right

; time stamp in units of 100ns; *1e-7 to convert to seconds
if n_elements(timestamp) ge 0 then begin
  timestamp = ulon64arr(nz)
  readu, 11, timestamp
  dt = 1e-7*(timestamp[nz-1] - timestamp[0])
  Print, 'Frame rate [FPs]: ', (nz - 1)/dt

endif


close, 11

if keyword_set(fits) then begin
  l = strpos(filename, '.ser')
  if (l lt 0) then l = strpos(filename, '.SER')
  fitsname =  strmid(filename,0,l)+'.fits' 
  sxhmake, img, 1, header  ; create header
  if keyword_set(text) then begin ; add info from the text file
      rdtext, filename+'.txt', textarr
      keywords = textarr[0,*]
      values = textarr[1,*]
      sxaddpar, header, 'BINNING', fix(values[3])  
      sxaddpar, header, 'OFFSET',  fix(values[4])
      sxaddpar, header, 'GAIN',  fix(values[10])
      sxaddpar, header, 'EXPOSURE', values[8]  
      sxaddpar, header, 'FORMAT', values[7]  
      sxaddpar, header, 'STARTX',  fix(values[14])
      sxaddpar, header, 'STARTY',  fix(values[15]) 
  endif
  print, 'Writing to FITS file '+fitsname
;  stop
  writefits, fitsname, img, header
endif
  
; stop
end
;-----------------------------------------------------
; time regularity
pro timing, timestamp

t = 1e-7*(timestamp - timestamp[0])   ; time in seconds

dt = 1e3*(t - shift(t,1)) ; frame time in ms
dt[0] = dt[1]
n = n_elements(t)

tmean = (t[n-1] - t[0])/(n-1)
print, 'Mean frame time: ', tmean
dt0  = median(dt)

bad = where(dt lt 0.5*dt0, nbad)
delayed = where(dt gt 1.5*dt0, ndel)
print, 'Total, Missed, delayed frames: ', n, nbad, ndel
plot, dt, psym=6, xtitle='Frame number', ytitle='Frame time [ms]'  

stop

x = findgen(n)
res = linfit(x,t)
dt1 = t - res[0]  - res[1]*x
plot, 1e3*dt1, xtitle='Frame number', ytitle='Time offset [ms]'

stop

end
;-----------------------------------------------------
; Test for presence of identical frames in the image cube
pro lostframes, img 

  n = n_elements(img[0,0,*])
  rms = fltarr(n-1)
  for i=0,n-2 do rms[i] = variance( img[*,*,i] - img[*,*,i+1])
  rms = sqrt(rms)
  meanrms = total(rms)/(n-1)
  bad = where(rms lt 0.1*meanrms, ns)
  print, 'Total frames, lost: ', n, ns
  if (ns gt 0) then print, 'Lost frames: ', bad+1
  
;  stop
end
;-----------------------------------------------------
; Convert all SER files to fits
pro allser, dir

  files = findfile(dir+'*.SER', count=n)
  if (n eq 0) then begin
      print, 'No SER files found. .C to return'
      stop
      return
      endif

  print, 'Found files: ',n

  for i=0,n-1 do begin 
      rdser, files[i], img, /fits, /text
      s=size(img) 
      nz = s[3]
;      imav = total(float(img),3)/nz
;      l = strpos(files[i], '.SER')
;      fitsname =  strmid(files[i],0,l)+'_av.fits' 
;      writefits,fitsname, imav
   endfor
  

end
;-----------------------------------------------------
;-----------------------------------------------------
