; read .prof and .stm files produced by RINGSS python code to dat structure
; input: file name without extension
; see allstm.pro   
pro readstm, file, dat, silent=silent


files = findfile(file,count=n)
if (n eq 0) then begin 
  print, 'Not found file '+file
  dat = -1
  return
endif

nmax= 3000  ; max number of records in one file
mmax= 20    ; fixed parameter
nsect = 8   ; fixed parameter
eladu = 0.36  ; for G=200 default electrons per ADU

impar = {backgr:0.,flux:0.,fluxvar:0.,rad:0.,rwidth:0., $
       xc:0.,yc:0.,xcvar:0.,ycvar:0.,coma:0.,angle:0., $
       contrast:0.,noisepar:fltarr(4)}
meancoef = fltarr(14)  ; mean radii and m=1,2,3 for aberrations
el={fname:'',jd:0D0,hr:0,bv:0.,zen:0.,texp:1e-3,gain:0.,nx:64,nz:2000, impar:impar, $
  flux:0.,rad:0.,rrms:0.,pow:fltarr(mmax+1), cov:fltarr(mmax+1), anoise:0.,rnoise:0., $
   prof:fltarr(8), see:0.,fsee:0.,noise:0.,scint:0.,rmserr:0.,see2:0.,rnoise1:0.,wind:0.,tau0:0.,mcoef:meancoef}

   dat = replicate(el,nmax)

get_lun, lun
openr, lun, file

tag = 'M'
k0=4
a = ''
k = 0  ; record counter
nitems = k0+16+2*(mmax+1) + 16 ; number of elements in each healthy M-type line

; main loop
while not eof(lun) do begin
  readf, lun, a
  b = strsplit(a, ',', /extract, /preserve_null)
  n = n_elements(b)
  if (n lt nitems) then continue ; skip the line
  if (b[2] eq '') then continue ; no star

  label = b[0]  ; tag  datetime
  if (label ne tag) then continue ; only labeled lines

  datetime = b[1]
  if not keyword_set(silent) then print, datetime
  yy = fix(strmid(datetime,0,4))
  mm = fix(strmid(datetime,5,2))
  dd = fix(strmid(datetime,8,2))
  hh = fix(strmid(datetime,11,2))
  min = fix(strmid(datetime,14,2))
  sec = fix(strmid(datetime,17,2))    
  jdcnv, yy, mm, dd, hh+min/60. + sec/3600., jd
  dat[k].jd = jd - 2400000.D0

  impar1 = float(b[k0:k0+11])
  rad = impar1[3]

  dat[k].fname = datetime
  dat[k].hr = fix(b[2])
  for j=0,11 do dat[k].impar.(j) = impar1[j]
  dat[k].impar.noisepar = float(b[k0+12:k0+15])
  m = mmax+1
  k1 = k0+16
  dat[k].pow = float(b[k1:k1+m-1])
  dat[k].cov =  float(b[k1+m:k1+2*m-1])
  dat[k].flux = impar1[1]*eladu  ; flux in electrons
  dat[k].rad = rad
  k2 = k1+2*m  ; 59  

  dat[k].rnoise = float(b[k2])
  dat[k].rrms = sqrt(float(b[k2+1]))
  dat[k].mcoef = float(b[k2+2:k2+2+13])
  k++
endwhile
close, lun
free_lun, lun
 
print, k, ' records found'
dat = dat[0:k-1]
end

;-------------------------------------------------------------------------
; Add profile information to the dat structure
pro readprof, file, dat

ndat = n_elements(dat)
if (ndat le 1) then begin 
   print, 'Empty dat structure?'
   return 
endif

files = findfile(file,count=nf)
if (nf eq 0) then begin 
  print, 'Not found file '+file
  return
endif

; Date,Star,Zen,Flux, See2,See,Fsee,Wind,tau0,theta0,  scint,rms, prof*8 [E-13]
a=''
get_lun, lun
openr, lun, file
k = 0
kbad = 0  ; not found in dat

while not eof(lun) do begin
  readf, lun, a
  b = strsplit(a, ',', /extract)
  datetime = b[0]
  s = where(datetime eq dat.fname, ns)
  if (ns eq 0) then begin 
    print, 'Not found in dat: '+datetime
    kbad++
    continue
  endif
;  stop, 'debug'
;  dat[s].hr = fix(b[1])
  dat[s].zen = float(b[2])
  dat[s].see2 = float(b[4]) 
  dat[s].see = float(b[5])
  dat[s].fsee = float(b[6])
  dat[s].wind =  float(b[7])
  dat[s].tau0 =  float(b[8])
  dat[s].scint =  float(b[10])
  dat[s].rmserr =  float(b[11])
  dat[s].prof =  float(b[12:19])
  k++
endwhile
close, lun
free_lun, lun

print, 'Records in dat and prof: ', ndat, k
print, 'Unmatched records: ', kbad

;stop

end
;-------------------------------------------------------------------------
; read several .stm and .prof files into a dat structure
pro readlist, dir, list

nmax = 500  ; max number of files
fnames = strarr(nmax)

close, /all
get_lun, lun
a=''
k = 0
openr, lun, list

while not eof(lun) do begin 
  readf, lun, a
  fnames[k] = trim(a)
  k++
endwhile
close, lun
free_lun, lun
fnames = fnames[0:k-1]
print, k, ' files in the list, .c to continue'


stop

readstm, dir+fnames[0]+'.stm', dat, /silent ; get .stm for the first night
if (n_elements(dat) le 1) then begin
   print, 'Failed to read the first night!'
   exit
endif
readprof, dir+fnames[0]+'.prof', dat  ; add profiles

for i=1,k-1 do begin
  print, '>>>> feading '+fnames[i]
  readstm, dir+fnames[i]+'.stm', tmp, /silent
  if (n_elements(tmp) le 1) then continue  ; skip
  readprof, dir+fnames[i]+'.prof', tmp
  dat = [dat,tmp]
endfor
n = n_elements(dat)
print, 'Done! Total points: ', n
stop

save, fi='alldat.idl', dat

end
;-------------------------------------------------------------------------
; Remove wrong points
pro filterdat

radtol = 0.05  ; radius tolerance
maxsee = 4.
rmsmax = 0.3

restore, 'alldat.idl' ; dat
n = n_elements(dat)

rad =  median(dat.rad)
print, 'Median ring radius [pix]: ', rad
good = where((dat.rad ge rad*(1-radtol)) and (dat.rad le rad*(1+radtol)), ngood)
dat = dat[good]

good = where((dat.see le maxsee) and (dat.see2 le maxsee), ngood)
dat = dat[good]
print, 'Remaining points, %: ', ngood, float(ngood)/n
good = where(dat.rmserr lt rmsmax, ngood)
dat = dat[good]

stop
save, fi='alldat.idl', dat

end
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------
;-------------------------------------------------------------------------


