; Generation of complex amplitude resulting from 2 turbulent layers
; May 30, 2006. For 1 layer only set fhigh=0
;-------------INPUT PARAMETERS: ------------------------
; NGRID - half-size of simulation grid, pixels (512 recommended)
; PIXEL - grid pixel, meters
; LAMBDA - wavelength, m
; R0 - Fried parameter at 500nm, m
; FHIGH - fraction of high layer
; ZLOW - distance to low turbulent layer, m
; ZHIGH - distance to high turbulent layer, m
; Example: simatm, 512, 0.01, 0.5e-6, 0.1, 0.05, 0, 1e4
;---------------OUTPUT PARAMETERS ----------------------
; complex amplitude saved in atm.idl
;-------------------------------------------------------
pro simatm, ngrid,pixel,lambda,r0,fhigh,zlow,zhigh, seed0=seed0
;
size = 2*ngrid*pixel
;

tint0 = r0^(-5./3.)/0.423*(0.5*lambda/!pi)^(2)
see = (tint0/6.83e-13)^0.6
;tint0 = 6.8e-13*see^(5./3.) ; total turbulence integral, m^1/3
tint1 = tint0*fhigh          ; high-layer integral
tint2 = tint0*(1.-fhigh)     ; low-layer integral

if (zlow gt zhigh) then return

r01 = (0.423*(0.5*lambda/!pi)^(-2)*tint1)^(-3./5.) ; Fried parameter, high layer
r02 = (0.423*(0.5*lambda/!pi)^(-2)*tint2)^(-3./5.) ; Fried par. low layer

  print, 'Grid size,m ',size
  print, 'Fried parameters,m [low,high] ', r02,r01
  print, 'Integrals,m^1/3 ', tint2,tint1
  print, 'Altitudes,m ',zlow,zhigh

fact1 = sqrt(0.023)*(size/r01)^(5./6.)  ; for phase simulation
if fhigh eq 0. then fact1 = 0.
fact2 = sqrt(0.023)*(size/r02)^(5./6.)  ; for phase simulation
;

r = shift(dist(2*ngrid),ngrid,ngrid) ; radial distance from the grid center, pixels
r[ngrid,ngrid] = 1.e-3
r = double(r)

; the Fresnel filters 
farg = !dpi*lambda/(size^2)*r^2

; Use old seed if specified, otherwise generate one
if keyword_set(seed0) then seed=seed0 else x = randomn(seed)
    
if (fhigh gt 0.) then begin 
; simulate turbulence in high layer
  print, 'Simulating the high screen...'
  tmp = fact1*r^(-11./6.)*dcomplex(randomn(seed,2*ngrid,2*ngrid,/double),randomn(seed,2*ngrid,2*ngrid,/double)) 
  tmp[ngrid,ngrid] = dcomplex(0.,0.)
; simulate phase
  tmp = float(shift(fft(shift(tmp,ngrid,ngrid),/inverse),ngrid,ngrid))
  u1=dcomplex(cos(tmp),sin(tmp))      ; complex amplitude at turbulent layer1

; ------ propagate wavefront to low layer  -----------------
if zhigh-zlow gt 0. then begin
   print, 'Propagating to low layer...'
   tmp = dcomplex(cos(farg*(zhigh-zlow)), -sin(farg*(zhigh-zlow)))* shift(fft(shift(u1,ngrid,ngrid)), ngrid, ngrid)
     u1 = shift(fft(shift(tmp,ngrid,ngrid),/inverse), ngrid, ngrid)
   endif
endif else u1=1.d0

;stop ; test propagation

; simulate turbulence in low layer
  print, 'Simulating the low screen...'
tmp = fact2*r^(-11./6.)*dcomplex(randomn(seed,2*ngrid,2*ngrid,/double),randomn(seed,2*ngrid,2*ngrid,/double)) 
tmp[ngrid,ngrid] = dcomplex(0.,0.)
tmp = float(shift(fft(shift(tmp,ngrid,ngrid),/inverse),ngrid,ngrid))
u1 *= dcomplex(cos(tmp),sin(tmp))      ; complex amplitude at low turbulent layer 

; --------- propagate to ground -----------------
print, 'Propagating to ground...'
tmp =   dcomplex(cos(farg*zlow), -sin(farg*zlow))*shift(fft(shift(u1,ngrid,ngrid)), ngrid, ngrid)
u1 = shift(fft(shift(tmp,ngrid,ngrid),/inverse), ngrid, ngrid)


; if the same variable is given on input as z, zlow is not saved!

    ;save, filename='atm.idl', u1,ngrid,pixel,lambda,see,r0,fhigh,zhigh
save, filename='atm.idl', u1,ngrid,pixel,lambda,see,r0,fhigh,zhigh,zlow

  tvscl, (abs(u1)^2)

; compare to Rytov number  
  scind = total( (abs(u1)^2 - 1.)^2)/ (2D0*ngrid)^2 
  rytov = 19.22*lambda^(-7./6.)*(zlow^(5./6.)*tint2 + zhigh^(5./6.)*tint1)
  print, 'Rytov variance, simulated: ', rytov, scind
  
;stop

end
;-------------------------------------------------------
