; compute angular spectrum and radial variance
; Outputs: angular power spectrum, diff. rms motion [pix] and noise rms [pix]
pro statmom, coef, par, pow, rrms, rnoise, seeing=seeing, cov=cov

m = par.mmax
nsect = par.nsect
s = size(coef)
ncoef = s[1] & nz = s[2]
if (ncoef ne 2*nsect + 2*m +2) then begin
   print, 'Parameters do not match!'
   stop
   return
endif   

; compute radius variance
dr = coef[0:nsect/2-1,*] + coef[nsect/2:nsect-1,*]  ; 4 DIMM signals
drvar = fltarr(nsect/2)
for i=0,nsect/2-1 do drvar[i] = variance(dr[i,*])

; evaluate noise
ddr = dr - shift(dr,0,1)
drnoise = fltarr(nsect/2)
for i=0,nsect/2-1 do drnoise[i] = 0.5*variance(ddr[i,*])
;meanrvar = mean(drvar - drnoise) ; average 4 DIMM signals
meanrvar = mean(drvar) ; average 4 DIMM signals

rrms = sqrt(meanrvar)
rnoise = sqrt(mean(drnoise))
				     
; compute seeing
if keyword_set(seeing) then begin
  rwt = ringweight(0.,par.d, par.eps, par.pdist, par.lambda)
  meanvarrad = meanrvar*(par.pixel/206265.)^2 ; diff. motion in radians
  lamr0 = (meanvarrad/4./rwt)^0.6 *(par.d/par.lambda)^(1./5.)
  seeing = 206265.*lamr0*0.98 ; seeing in arcsec
  print, 'Estimated seeing [arcsec]: ', seeing
endif ; seeing

acoef = coef[2*nsect:ncoef-1,*]
meancoef = total(acoef,2)/nz
varcoef =  total(acoef^2,2)/nz - meancoef^2


;stop
tmp = (acoef*shift(acoef,0,1))[*,1:nz-1]
covar = total(tmp,2)/(nz-1) - meancoef^2

cov = covar[0:m] + covar[m+1:2*m+1]

pow = varcoef[0:m] + varcoef[m+1:2*m+1]
arg = findgen(m+1)

 !p.charsize=1.5
 plot, arg, pow, /ylog, xtitle='m', ytitle='Power'
 oplot, arg, cov, li=2
 
; stop
; rho = cov/pow
;  plot, arg, pow, /ylog, xtitle='m', ytitle='Correlation coefficient'


end
